import numpy as np


# Input: 1D list or numpy array
def get_frequencies(data, return_unique_and_counts=False):
    if isinstance(data, list):
        data = np.array(data)
    elif isinstance(data, np.ndarray):
        pass
    else:
        raise ValueError('The input data should be 1D list or numpy array.')
    (unique, counts) = np.unique(data, return_counts=True)
    frequencies = np.asarray((unique, counts)).T
    if return_unique_and_counts:
        return frequencies, unique, counts
    else:
        return frequencies



import matplotlib.pyplot as plt
import numpy as np


# === Plot horizontal bar ===


# Input: array like
def plot_horizontal_bar(y_values, y_indices, x_label=None, title=None, show=True):
    fig, ax = plt.subplots()
    y_ticks = np.arange(len(y_indices))
    ax.barh(y=y_ticks, width=y_values, align='center')
    ax.set_yticks(y_ticks)
    ax.set_yticklabels(y_indices)
    ax.invert_yaxis()
    if x_label:
        ax.set_xlabel(x_label)
    if title:
        ax.set_title(title)
    if show:
        plt.show()


# Practicing:
'''
# Plot how many images each rater has rated
def plot_how_many_images_each_rater_has_rated(df_record):
    # rater_to_num = df_attrs['hp'].groupby('name').size()
    rater_to_num = df_record.groupby('name').size()
    plt.rcdefaults()
    fig, ax = plt.subplots()
    ax.barh(y=np.arange(len(rater_to_num.index.values)), width=rater_to_num.values, align='center')
    ax.set_yticks(np.arange(len(rater_to_num.index.values)))
    ax.set_yticklabels(rater_to_num.index.values)
    ax.invert_yaxis()
    ax.set_xlabel('Number of images has rated')
    ax.set_title('How many images each rater has rated?')
    plt.show()

plot_how_many_images_each_rater_has_rated(df_record)
'''



from PIL import Image
import numpy as np

# === Get image information ===


VALID_GET_IMG_INPUT_MODE = ['file_path', 'array', 'PIL_Image']
VALID_GET_IMG_RETURN_MODE = ['array', 'PIL_Image']


# Parameters:
#     input_mode:
#         - 'file_path': Get img information by opening the img file.
#         - 'array': Input the image in numpy array format.
#         - 'PIL_Image': Input the image in PIL Image format.
#     return_mode:
#         - 'array': Return the image in numpy array format.
#         - 'PIL_Image': Return the image in PIL Image format.
#     img:
#         Whether they should be given depends on the mode you choose.
#         e.g. If input mode is 'array', img should be a numpy array which represents the image.
def get_img(img,
            input_mode='file_path',
            return_mode='array'
            ):
    # Check if the two mode valid
    if input_mode not in VALID_GET_IMG_INPUT_MODE or return_mode not in VALID_GET_IMG_RETURN_MODE:
        raise ValueError('Invalid input mode or invalid return_mode.')
    # Get image
    img = img
    if input_mode == 'file_path':
        img = Image.open(img)
    elif input_mode == 'array':
        img = Image.fromarray(img)
    elif input_mode == 'PIL_Image':
        pass
    # Return image
    if return_mode == 'array':
        return np.array(img)
    elif return_mode == 'PIL_Image':
        return img


def remove_transparency(pil_img):
    # Remove transparency
    if len(pil_img.split()) > 3:
        # img.load()  # required for png.split()
        background = Image.new("RGB", pil_img.size, (255, 255, 255))
        background.paste(pil_img, mask=pil_img.split()[3])  # 3 is the alpha channel
        pil_img = background
    return pil_img


import yaml

'''
How to install the python yaml package--"pyyaml"
1. pip install pyyaml
2. import yaml
'''


class YAML:
    def __init__(self, yaml_path=None):
        # Initialize
        self.__data = None
        # Load data depending on whether the param is given
        if yaml_path is not None:
            self.load(yaml_path=yaml_path)

    def load(self, yaml_path):
        # Load yaml data (as a dict)
        with open(yaml_path) as yaml_file:
            self.__data = yaml.load(yaml_file, Loader=yaml.FullLoader)

    # Get the data dict
    def get_data(self):
        return self.__data




from random import randint
import numpy as np
from PIL import Image
# import wctools.image_wc.size_manipulation
from wctools.image.img_format import get_img


# from wctools.image.img_format import get_img


# from

# === Resize image with zero-padding ===


# Resize to specific width and height with zero-padding
# step 1. If the width or height is larger than target, remain the proportion,
#         and make the width and height <= target width and height.
# step 2. Fill zero (black) in unfilled region,
#         and make original region in the center.
#
# Parameters:
#     pil_img_mode:
#         - 'none'
#     input_mode & img:
#         See the function "get_img(...)" in "img_format.py" file for more information.
#     target_size: tuple (target_width, target_height):
#         The size you want your image be resized.
#     return_mode:
#         - 'array'
#         - 'PIL_Image'
def resize_with_zero_padding(img,
                             # pil_img_mode: str,
                             input_mode='file_path',
                             target_size=(100, 100),
                             return_mode='array'):
    # Get img
    img = get_img(img=img, input_mode=input_mode, return_mode='PIL_Image')

    # Fit width
    w, h = img.size
    target_w, target_h = target_size
    if w > target_w:
        img = img.resize((target_w, int(h * (target_w / w))))

    # Fit height
    w, h = img.size
    if h > target_h:
        img = img.resize((int(w * (target_h / h)), target_h))

    # Zero padding
    w, h = img.size
    w_padding = target_w - w
    h_padding = target_h - h
    img = img.crop(
        [-(w_padding // 2), -(h_padding // 2), w + w_padding - (w_padding // 2), h + h_padding - (h_padding // 2)])
    # The other way to add zero:
    # img_arr = np.array(img)
    # img_arr = np.concatenate([np.zeros((target_height, total_padding//2, 3), dtype="uint8"), \
    # img_arr, np.zeros((target_height, total_padding-total_padding//2, 3))], 1)
    # img_arr = img_arr.astype('uint8')
    # img = Image.fromarray(img_arr)

    # Return in specific format
    if return_mode == 'array':
        return np.array(img)
    elif return_mode == 'PIL_Image':
        return img
    else:
        raise ValueError('Invalid return format.')


# === Randomly Crop ===


# Parameters:
#     img & input_mode: See "get_img(...)" function in "img_format.py" for more information.
#     bbox_size: tuple (bbox_width, bbox_height)
#     min_size: tuple (min_width, min_height).
#         If w or h of img is smaller bbox_size's, the img will be resized in the same proportion.
#         Both w and h of min_size must be equivalent or larger than bbox_size's.
#     all_possible:
#         Determine if return all possible crop.
#         If this parameter is True, it'll return a list of object; otherwise, a single object.
#         What is the object's format depends on the parameter "return_mode".
#     return_mode: 'PIL_Image' or 'array'
#         Determine if the return is/are PIL Image(s) or numpy array(s).
#
# P.S.
#     Suggest to turn bbox_size smaller than min_size to get different crop every time.
def random_crop(img,
                bbox_size,
                input_mode='file_path',
                min_size='bbox_size',
                all_possible=False,
                return_mode='PIL_Image'
                ):
    # Get img
    img = get_img(img, input_mode=input_mode, return_mode='PIL_Image')

    # Check if the two size are in valid format and get width and height
    if type(bbox_size) is not tuple or (min_size != 'bbox_size' and type(min_size) is not tuple):
        raise ValueError('Parameter Error')
    (bbox_w, bbox_h) = bbox_size
    (min_w, min_h) = min_size if min_size != 'bbox_size' else bbox_size

    # Check if min_size < bbox_size
    if min_w < bbox_w or min_h < bbox_h:
        raise Exception("Height and width of min_size should be both larger than bbox_size's.")

    # Adjust size to at least width and height equivalent with min_size's
    w, h = img.size
    if w < min_w:
        img = img.resize((min_w, int(h * (min_w / w))))
    if h < min_h:
        img = img.resize((int(w * (min_h / h)), min_h))

    # Randomly crop
    w, h = img.size
    w_rand_len = w - bbox_w
    h_rand_len = h - bbox_h

    # random_tile = img.crop([left, upper, right, lower])
    # 0 - - - - - - > x
    # |
    # | Image.crop([start_x, start_y, end_x, end_y])
    # V
    # y
    if all_possible:
        all_possible_crop = []
        for h_num in range(h_rand_len + 1):
            img_cropped = None
            for w_num in range(w_rand_len + 1):
                left = w_num
                right = w_num + bbox_w
                upper = h_num
                lower = h_num + bbox_h
                img_cropped = img.crop([left, upper, right, lower])
            if return_mode == 'PIL_Image':
                all_possible_crop.append(img_cropped)
            elif return_mode == 'array':
                all_possible_crop.append(np.array(img_cropped))
        return all_possible_crop
    else:
        left = randint(0, w_rand_len)
        right = left + bbox_w
        upper = randint(0, h_rand_len)
        lower = upper + bbox_h
        img_cropped = img.crop([left, upper, right, lower])
        return img_cropped if return_mode == 'PIL_Image' else np.array(img_cropped)


# === Make image be square with zero-padding ===


# Parameters:
#     img & input_mode: See "get_img(...)" function in "img_format.py" for more information.
#     return_mode: 'array' or 'PIL_Image'
#         If return mode is PIL Image, it will be turned into a RGB image.
#     pil_mode: If the return mode is 'PIL_Image', you should give PIL mode.
#         Default is 'RGB'.
# P.S.
#     Image will be turned to RGB.
def make_square_image(img, input_mode='file_path', return_mode='PIL_Image'):
    img = get_img(img=img, input_mode=input_mode, return_mode='PIL_Image')
    w, h = img.size
    target_size = (w, w) if w > h else (h, h)
    return resize_with_zero_padding(img, input_mode='PIL_Image', target_size=target_size, return_mode=return_mode)


"""
Example:
make_square_image(input_mode='file_path', img='data/images/100489.jpg', return_mode='PIL_Image').show()
"""



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wctools",  # Replace with your own username
    version="0.2.20201224",
    author="LiaoWC",
    author_email="wcl.cs07@nctu.edu.tw",
    description="A small package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LiaoWC",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "opencv_python==4.4.0.46",
        "joblib==0.16.0",
        "matplotlib==3.3.3",
        "pandas==1.1.4",
        "Werkzeug==0.16.1",
        "Pillow==8.0.1",
        "scikit_learn==0.23.2",
        "seaborn==0.11.1"
    ]
)
